@@ -1,8 +1,8 @@
 /* Any copyright is dedicated to the Public Domain.
    http://creativecommons.org/publicdomain/zero/1.0/ */
 
-add_task(async function() {
-  registerCleanupFunction(function() {
+add_task(async function () {
+  registerCleanupFunction(function () {
     window.restore();
   });
   function isActive() {
@@ -10,6 +10,7 @@ add_task(async function() {
   }
 
   ok(isActive(), "Docshell should be active when starting the test");
+  ok(!document.hidden, "Top level window should be visible");
 
   info("Calling window.minimize");
   let promiseSizeModeChange = BrowserTestUtils.waitForEvent(
@@ -19,6 +20,7 @@ add_task(async function() {
   window.minimize();
   await promiseSizeModeChange;
   ok(!isActive(), "Docshell should be Inactive");
+  ok(document.hidden, "Top level window should be hidden");
 
   info("Calling window.restore");
   promiseSizeModeChange = BrowserTestUtils.waitForEvent(
@@ -43,4 +45,5 @@ add_task(async function() {
     await BrowserTestUtils.waitForEvent(window, "occlusionstatechange");
   }
   ok(isActive(), "Docshell should be active again");
+  ok(!document.hidden, "Top level window should be visible");
 });
