@@ -1,16 +1,16 @@
-ChromeUtils.defineModuleGetter(
-  this,
-  "TelemetryTestUtils",
-  "resource://testing-common/TelemetryTestUtils.jsm"
+ChromeUtils.defineESModuleGetters(this, {
+  TelemetryTestUtils: "resource://testing-common/TelemetryTestUtils.sys.mjs",
+});
+const { AttributionIOUtils } = ChromeUtils.importESModule(
+  "resource:///modules/AttributionCode.sys.mjs"
 );
-const { sinon } = ChromeUtils.import("resource://testing-common/Sinon.jsm");
 
 add_task(async function test_parse_error() {
   if (AppConstants.platform == "macosx") {
     // On macOS, the underlying data is the OS-level quarantine
     // database.  We need to start from nothing to isolate the cache.
-    const { MacAttribution } = ChromeUtils.import(
-      "resource:///modules/MacAttribution.jsm"
+    const { MacAttribution } = ChromeUtils.importESModule(
+      "resource:///modules/MacAttribution.sys.mjs"
     );
     let attributionSvc = Cc["@mozilla.org/mac-attribution;1"].getService(
       Ci.nsIMacAttributionService
@@ -38,20 +38,27 @@ add_task(async function test_parse_error() {
   );
 
   // Write an invalid file to trigger a decode error
-  await AttributionCode.deleteFileAsync();
-  AttributionCode._clearCache();
-  // Empty string is valid on macOS.
-  await AttributionCode.writeAttributionFile(
-    AppConstants.platform == "macosx" ? "invalid" : ""
-  );
-  result = await AttributionCode.getAttrDataAsync();
-  Assert.deepEqual(result, {}, "Should have failed to parse");
+  // Skip this for MSIX packages though - we can't write or delete
+  // the attribution file there, everything happens in memory instead.
+  if (
+    AppConstants.platform === "win" &&
+    !Services.sysinfo.getProperty("hasWinPackageId")
+  ) {
+    await AttributionCode.deleteFileAsync();
+    AttributionCode._clearCache();
+    // Empty string is valid on macOS.
+    await AttributionCode.writeAttributionFile(
+      AppConstants.platform == "macosx" ? "invalid" : ""
+    );
+    result = await AttributionCode.getAttrDataAsync();
+    Assert.deepEqual(result, {}, "Should have failed to parse");
 
-  // `assertHistogram` also ensures that `read_error` index 0 is 0
-  // as we should not have recorded telemetry from the previous `getAttrDataAsync` call
-  TelemetryTestUtils.assertHistogram(histogram, INDEX_DECODE_ERROR, 1);
-  // Reset
-  histogram.clear();
+    // `assertHistogram` also ensures that `read_error` index 0 is 0
+    // as we should not have recorded telemetry from the previous `getAttrDataAsync` call
+    TelemetryTestUtils.assertHistogram(histogram, INDEX_DECODE_ERROR, 1);
+    // Reset
+    histogram.clear();
+  }
 });
 
 add_task(async function test_read_error() {
@@ -62,7 +69,7 @@ add_task(async function test_read_error() {
   const histogram = Services.telemetry.getHistogramById(
     "BROWSER_ATTRIBUTION_ERRORS"
   );
-  const sandbox = sinon.createSandbox();
+
   // Delete the file to trigger a read error
   await AttributionCode.deleteFileAsync();
   AttributionCode._clearCache();
@@ -70,10 +77,19 @@ add_task(async function test_read_error() {
   histogram.clear();
 
   // Force the file to exist but then cause a read error
-  const exists = sandbox.stub(OS.File, "exists");
-  exists.resolves(true);
-  const read = sandbox.stub(OS.File, "read");
-  read.throws(() => new Error("read_error"));
+  let oldExists = AttributionIOUtils.exists;
+  AttributionIOUtils.exists = () => true;
+
+  let oldRead = AttributionIOUtils.read;
+  AttributionIOUtils.read = () => {
+    throw new Error("read_error");
+  };
+
+  registerCleanupFunction(() => {
+    AttributionIOUtils.exists = oldExists;
+    AttributionIOUtils.read = oldRead;
+  });
+
   // Try to read the file
   await AttributionCode.getAttrDataAsync();
 
@@ -82,5 +98,4 @@ add_task(async function test_read_error() {
 
   // Clear any existing telemetry
   histogram.clear();
-  sandbox.restore();
 });
