@@ -3,18 +3,12 @@
 
 "use strict";
 
-ChromeUtils.defineModuleGetter(
-  this,
-  "AddonManager",
-  "resource://gre/modules/AddonManager.jsm"
-);
-
-const { AddonTestUtils } = ChromeUtils.import(
-  "resource://testing-common/AddonTestUtils.jsm"
-);
-const { SearchTestUtils } = ChromeUtils.import(
-  "resource://testing-common/SearchTestUtils.jsm"
-);
+ChromeUtils.defineESModuleGetters(this, {
+  AddonManager: "resource://gre/modules/AddonManager.sys.mjs",
+  AddonTestUtils: "resource://testing-common/AddonTestUtils.sys.mjs",
+  SearchTestUtils: "resource://testing-common/SearchTestUtils.sys.mjs",
+  TelemetryTestUtils: "resource://testing-common/TelemetryTestUtils.sys.mjs",
+});
 
 const EXTENSION1_ID = "extension1@mozilla.com";
 const EXTENSION2_ID = "extension2@mozilla.com";
@@ -24,16 +18,79 @@ const DEFAULT_SEARCH_SETTING_NAME = "defaultSearch";
 AddonTestUtils.initMochitest(this);
 SearchTestUtils.init(this);
 
-const DEFAULT_ENGINE_NAME = "basic";
-const ALTERNATE_ENGINE_NAME = "Simple Engine";
-const ALTERNATE2_ENGINE_NAME = "another";
+const DEFAULT_ENGINE = {
+  id: "basic",
+  name: "basic",
+  loadPath: "[addon]basic@search.mozilla.org",
+  submissionUrl:
+    "https://mochi.test:8888/browser/browser/components/search/test/browser/?search=&foo=1",
+};
+const ALTERNATE_ENGINE = {
+  id: "simple",
+  name: "Simple Engine",
+  loadPath: "[addon]simple@search.mozilla.org",
+  submissionUrl: "https://example.com/?sourceId=Mozilla-search&search=",
+};
+const ALTERNATE2_ENGINE = {
+  id: "simple",
+  name: "another",
+  loadPath: "",
+  submissionUrl: "",
+};
 
 async function restoreDefaultEngine() {
-  let engine = Services.search.getEngineByName(DEFAULT_ENGINE_NAME);
-  await Services.search.setDefault(engine);
+  let engine = Services.search.getEngineByName(DEFAULT_ENGINE.name);
+  await Services.search.setDefault(
+    engine,
+    Ci.nsISearchService.CHANGE_REASON_UNKNOWN
+  );
 }
 
-add_task(async function setup() {
+function clearTelemetry() {
+  Services.telemetry.clearEvents();
+  Services.fog.testResetFOG();
+}
+
+async function checkTelemetry(source, prevEngine, newEngine) {
+  TelemetryTestUtils.assertEvents(
+    [
+      {
+        object: "change_default",
+        value: source,
+        extra: {
+          prev_id: prevEngine.id,
+          new_id: newEngine.id,
+          new_name: newEngine.name,
+          new_load_path: newEngine.loadPath,
+          // Telemetry has a limit of 80 characters.
+          new_sub_url: newEngine.submissionUrl.slice(0, 80),
+        },
+      },
+    ],
+    { category: "search", method: "engine" }
+  );
+
+  let snapshot = await Glean.searchEngineDefault.changed.testGetValue();
+  delete snapshot[0].timestamp;
+  Assert.deepEqual(
+    snapshot[0],
+    {
+      category: "search.engine.default",
+      name: "changed",
+      extra: {
+        change_source: source,
+        previous_engine_id: prevEngine.id,
+        new_engine_id: newEngine.id,
+        new_display_name: newEngine.name,
+        new_load_path: newEngine.loadPath,
+        new_submission_url: newEngine.submissionUrl,
+      },
+    },
+    "Should have received the correct event details"
+  );
+}
+
+add_setup(async function () {
   let searchExtensions = getChromeDir(getResolvedURI(gTestPath));
   searchExtensions.append("search-engines");
 
@@ -55,11 +112,13 @@ add_task(async function setup() {
 
 /* This tests setting a default engine. */
 add_task(async function test_extension_setting_default_engine() {
+  clearTelemetry();
+
   let ext1 = ExtensionTestUtils.loadExtension({
     manifest: {
       chrome_settings_overrides: {
         search_provider: {
-          name: ALTERNATE_ENGINE_NAME,
+          name: ALTERNATE_ENGINE.name,
           search_url: "https://example.com/?q={searchTerms}",
           is_default: true,
         },
@@ -73,29 +132,35 @@ add_task(async function test_extension_setting_default_engine() {
 
   is(
     (await Services.search.getDefault()).name,
-    ALTERNATE_ENGINE_NAME,
-    `Default engine is ${ALTERNATE_ENGINE_NAME}`
+    ALTERNATE_ENGINE.name,
+    `Default engine is ${ALTERNATE_ENGINE.name}`
   );
 
+  await checkTelemetry("addon-install", DEFAULT_ENGINE, ALTERNATE_ENGINE);
+
+  clearTelemetry();
+
   await ext1.unload();
 
   is(
     (await Services.search.getDefault()).name,
-    DEFAULT_ENGINE_NAME,
-    `Default engine is ${DEFAULT_ENGINE_NAME}`
+    DEFAULT_ENGINE.name,
+    `Default engine is ${DEFAULT_ENGINE.name}`
   );
+
+  await checkTelemetry("addon-uninstall", ALTERNATE_ENGINE, DEFAULT_ENGINE);
 });
 
 /* This tests what happens when the engine you're setting it to is hidden. */
 add_task(async function test_extension_setting_default_engine_hidden() {
-  let engine = Services.search.getEngineByName(ALTERNATE_ENGINE_NAME);
+  let engine = Services.search.getEngineByName(ALTERNATE_ENGINE.name);
   engine.hidden = true;
 
   let ext1 = ExtensionTestUtils.loadExtension({
     manifest: {
       chrome_settings_overrides: {
         search_provider: {
-          name: ALTERNATE_ENGINE_NAME,
+          name: ALTERNATE_ENGINE.name,
           search_url: "https://example.com/?q={searchTerms}",
           is_default: true,
         },
@@ -109,7 +174,7 @@ add_task(async function test_extension_setting_default_engine_hidden() {
 
   is(
     (await Services.search.getDefault()).name,
-    DEFAULT_ENGINE_NAME,
+    DEFAULT_ENGINE.name,
     "Default engine should have remained as the default"
   );
   is(
@@ -122,8 +187,8 @@ add_task(async function test_extension_setting_default_engine_hidden() {
 
   is(
     (await Services.search.getDefault()).name,
-    DEFAULT_ENGINE_NAME,
-    `Default engine is ${DEFAULT_ENGINE_NAME}`
+    DEFAULT_ENGINE.name,
+    `Default engine is ${DEFAULT_ENGINE.name}`
   );
   engine.hidden = false;
 });
@@ -138,7 +203,11 @@ add_task(async function test_extension_setting_default_engine_external() {
   async function startExtension(win = window) {
     let extension = ExtensionTestUtils.loadExtension({
       manifest: {
-        applications: {
+        icons: {
+          48: "icon.png",
+          96: "icon@2x.png",
+        },
+        browser_specific_settings: {
           gecko: {
             id: EXTENSION1_ID,
           },
@@ -151,6 +220,10 @@ add_task(async function test_extension_setting_default_engine_external() {
           },
         },
       },
+      files: {
+        "icon.png": "",
+        "icon@2x.png": "",
+      },
       useAddonManager: "temporary",
     });
 
@@ -170,18 +243,25 @@ add_task(async function test_extension_setting_default_engine_external() {
 
   // First time around, don't accept the default engine.
   let { panel, extension } = await startExtension();
+  ok(
+    panel.getAttribute("icon").endsWith("/icon.png"),
+    "expected custom icon set on the notification"
+  );
+
   panel.secondaryButton.click();
 
   await TestUtils.topicObserved("webextension-defaultsearch-prompt-response");
 
   is(
     (await Services.search.getDefault()).name,
-    DEFAULT_ENGINE_NAME,
+    DEFAULT_ENGINE.name,
     "Default engine was not changed after rejecting prompt"
   );
 
   await extension.unload();
 
+  clearTelemetry();
+
   // Do it again, this time accept the prompt.
   ({ panel, extension } = await startExtension());
 
@@ -194,6 +274,14 @@ add_task(async function test_extension_setting_default_engine_external() {
     "Default engine was changed after accepting prompt"
   );
 
+  await checkTelemetry("addon-install", DEFAULT_ENGINE, {
+    id: "other-Example Engine",
+    name: "Example Engine",
+    loadPath: "[addon]extension1@mozilla.com",
+    submissionUrl: "https://example.com/?q=",
+  });
+  clearTelemetry();
+
   // Do this twice to make sure we're definitely handling disable/enable
   // correctly.  Disabling and enabling the addon here like this also
   // replicates the behavior when an addon is added then removed in the
@@ -205,10 +293,22 @@ add_task(async function test_extension_setting_default_engine_external() {
 
   is(
     (await Services.search.getDefault()).name,
-    DEFAULT_ENGINE_NAME,
-    `Default engine is ${DEFAULT_ENGINE_NAME} after disabling`
+    DEFAULT_ENGINE.name,
+    `Default engine is ${DEFAULT_ENGINE.name} after disabling`
   );
 
+  await checkTelemetry(
+    "addon-uninstall",
+    {
+      id: "other-Example Engine",
+      name: "Example Engine",
+      loadPath: "[addon]extension1@mozilla.com",
+      submissionUrl: "https://example.com/?q=",
+    },
+    DEFAULT_ENGINE
+  );
+  clearTelemetry();
+
   let opened = promisePopupNotificationShown(
     "addon-webext-defaultsearch",
     window
@@ -224,11 +324,18 @@ add_task(async function test_extension_setting_default_engine_external() {
     `Default engine is ${NAME} after enabling`
   );
 
+  await checkTelemetry("addon-install", DEFAULT_ENGINE, {
+    id: "other-Example Engine",
+    name: "Example Engine",
+    loadPath: "[addon]extension1@mozilla.com",
+    submissionUrl: "https://example.com/?q=",
+  });
+
   await extension.unload();
 
   is(
     (await Services.search.getDefault()).name,
-    DEFAULT_ENGINE_NAME,
+    DEFAULT_ENGINE.name,
     "Default engine is reverted after uninstalling extension."
   );
 
@@ -243,7 +350,7 @@ add_task(async function test_extension_setting_default_engine_external() {
 
   is(
     (await Services.search.getDefault()).name,
-    DEFAULT_ENGINE_NAME,
+    DEFAULT_ENGINE.name,
     "Default engine is unchanged when prompt is dismissed"
   );
 
@@ -257,7 +364,7 @@ add_task(async function test_extension_setting_multiple_default_engine() {
     manifest: {
       chrome_settings_overrides: {
         search_provider: {
-          name: ALTERNATE_ENGINE_NAME,
+          name: ALTERNATE_ENGINE.name,
           search_url: "https://example.com/?q={searchTerms}",
           is_default: true,
         },
@@ -270,7 +377,7 @@ add_task(async function test_extension_setting_multiple_default_engine() {
     manifest: {
       chrome_settings_overrides: {
         search_provider: {
-          name: ALTERNATE2_ENGINE_NAME,
+          name: ALTERNATE2_ENGINE.name,
           search_url: "https://example.com/?q={searchTerms}",
           is_default: true,
         },
@@ -284,8 +391,8 @@ add_task(async function test_extension_setting_multiple_default_engine() {
 
   is(
     (await Services.search.getDefault()).name,
-    ALTERNATE_ENGINE_NAME,
-    `Default engine is ${ALTERNATE_ENGINE_NAME}`
+    ALTERNATE_ENGINE.name,
+    `Default engine is ${ALTERNATE_ENGINE.name}`
   );
 
   await ext2.startup();
@@ -293,24 +400,24 @@ add_task(async function test_extension_setting_multiple_default_engine() {
 
   is(
     (await Services.search.getDefault()).name,
-    ALTERNATE2_ENGINE_NAME,
-    `Default engine is ${ALTERNATE2_ENGINE_NAME}`
+    ALTERNATE2_ENGINE.name,
+    `Default engine is ${ALTERNATE2_ENGINE.name}`
   );
 
   await ext2.unload();
 
   is(
     (await Services.search.getDefault()).name,
-    ALTERNATE_ENGINE_NAME,
-    `Default engine is ${ALTERNATE_ENGINE_NAME}`
+    ALTERNATE_ENGINE.name,
+    `Default engine is ${ALTERNATE_ENGINE.name}`
   );
 
   await ext1.unload();
 
   is(
     (await Services.search.getDefault()).name,
-    DEFAULT_ENGINE_NAME,
-    `Default engine is ${DEFAULT_ENGINE_NAME}`
+    DEFAULT_ENGINE.name,
+    `Default engine is ${DEFAULT_ENGINE.name}`
   );
 });
 
@@ -322,7 +429,7 @@ add_task(
       manifest: {
         chrome_settings_overrides: {
           search_provider: {
-            name: ALTERNATE_ENGINE_NAME,
+            name: ALTERNATE_ENGINE.name,
             search_url: "https://example.com/?q={searchTerms}",
             is_default: true,
           },
@@ -335,7 +442,7 @@ add_task(
       manifest: {
         chrome_settings_overrides: {
           search_provider: {
-            name: ALTERNATE2_ENGINE_NAME,
+            name: ALTERNATE2_ENGINE.name,
             search_url: "https://example.com/?q={searchTerms}",
             is_default: true,
           },
@@ -349,8 +456,8 @@ add_task(
 
     is(
       (await Services.search.getDefault()).name,
-      ALTERNATE_ENGINE_NAME,
-      `Default engine is ${ALTERNATE_ENGINE_NAME}`
+      ALTERNATE_ENGINE.name,
+      `Default engine is ${ALTERNATE_ENGINE.name}`
     );
 
     await ext2.startup();
@@ -358,24 +465,24 @@ add_task(
 
     is(
       (await Services.search.getDefault()).name,
-      ALTERNATE2_ENGINE_NAME,
-      `Default engine is ${ALTERNATE2_ENGINE_NAME}`
+      ALTERNATE2_ENGINE.name,
+      `Default engine is ${ALTERNATE2_ENGINE.name}`
     );
 
     await ext1.unload();
 
     is(
       (await Services.search.getDefault()).name,
-      ALTERNATE2_ENGINE_NAME,
-      `Default engine is ${ALTERNATE2_ENGINE_NAME}`
+      ALTERNATE2_ENGINE.name,
+      `Default engine is ${ALTERNATE2_ENGINE.name}`
     );
 
     await ext2.unload();
 
     is(
       (await Services.search.getDefault()).name,
-      DEFAULT_ENGINE_NAME,
-      `Default engine is ${DEFAULT_ENGINE_NAME}`
+      DEFAULT_ENGINE.name,
+      `Default engine is ${DEFAULT_ENGINE.name}`
     );
   }
 );
@@ -387,7 +494,7 @@ add_task(async function test_user_changing_default_engine() {
     manifest: {
       chrome_settings_overrides: {
         search_provider: {
-          name: ALTERNATE_ENGINE_NAME,
+          name: ALTERNATE_ENGINE.name,
           search_url: "https://example.com/?q={searchTerms}",
           is_default: true,
         },
@@ -401,12 +508,15 @@ add_task(async function test_user_changing_default_engine() {
 
   is(
     (await Services.search.getDefault()).name,
-    ALTERNATE_ENGINE_NAME,
-    `Default engine is ${ALTERNATE_ENGINE_NAME}`
+    ALTERNATE_ENGINE.name,
+    `Default engine is ${ALTERNATE_ENGINE.name}`
   );
 
-  let engine = Services.search.getEngineByName(ALTERNATE2_ENGINE_NAME);
-  await Services.search.setDefault(engine);
+  let engine = Services.search.getEngineByName(ALTERNATE2_ENGINE.name);
+  await Services.search.setDefault(
+    engine,
+    Ci.nsISearchService.CHANGE_REASON_UNKNOWN
+  );
   // This simulates the preferences UI when the setting is changed.
   ExtensionSettingsStore.select(
     ExtensionSettingsStore.SETTING_USER_SET,
@@ -418,8 +528,8 @@ add_task(async function test_user_changing_default_engine() {
 
   is(
     (await Services.search.getDefault()).name,
-    ALTERNATE2_ENGINE_NAME,
-    `Default engine is ${ALTERNATE2_ENGINE_NAME}`
+    ALTERNATE2_ENGINE.name,
+    `Default engine is ${ALTERNATE2_ENGINE.name}`
   );
   restoreDefaultEngine();
 });
@@ -429,14 +539,14 @@ add_task(async function test_user_changing_default_engine() {
 add_task(async function test_user_change_with_disabling() {
   let ext1 = ExtensionTestUtils.loadExtension({
     manifest: {
-      applications: {
+      browser_specific_settings: {
         gecko: {
           id: EXTENSION1_ID,
         },
       },
       chrome_settings_overrides: {
         search_provider: {
-          name: ALTERNATE_ENGINE_NAME,
+          name: ALTERNATE_ENGINE.name,
           search_url: "https://example.com/?q={searchTerms}",
           is_default: true,
         },
@@ -450,12 +560,15 @@ add_task(async function test_user_change_with_disabling() {
 
   is(
     (await Services.search.getDefault()).name,
-    ALTERNATE_ENGINE_NAME,
-    `Default engine is ${ALTERNATE_ENGINE_NAME}`
+    ALTERNATE_ENGINE.name,
+    `Default engine is ${ALTERNATE_ENGINE.name}`
   );
 
-  let engine = Services.search.getEngineByName(ALTERNATE2_ENGINE_NAME);
-  await Services.search.setDefault(engine);
+  let engine = Services.search.getEngineByName(ALTERNATE2_ENGINE.name);
+  await Services.search.setDefault(
+    engine,
+    Ci.nsISearchService.CHANGE_REASON_UNKNOWN
+  );
   // This simulates the preferences UI when the setting is changed.
   ExtensionSettingsStore.select(
     ExtensionSettingsStore.SETTING_USER_SET,
@@ -465,8 +578,8 @@ add_task(async function test_user_change_with_disabling() {
 
   is(
     (await Services.search.getDefault()).name,
-    ALTERNATE2_ENGINE_NAME,
-    `Default engine is ${ALTERNATE2_ENGINE_NAME}`
+    ALTERNATE2_ENGINE.name,
+    `Default engine is ${ALTERNATE2_ENGINE.name}`
   );
 
   let disabledPromise = awaitEvent("shutdown", EXTENSION1_ID);
@@ -476,8 +589,8 @@ add_task(async function test_user_change_with_disabling() {
 
   is(
     (await Services.search.getDefault()).name,
-    ALTERNATE2_ENGINE_NAME,
-    `Default engine is ${ALTERNATE2_ENGINE_NAME}`
+    ALTERNATE2_ENGINE.name,
+    `Default engine is ${ALTERNATE2_ENGINE.name}`
   );
 
   let processedPromise = awaitEvent("searchEngineProcessed", EXTENSION1_ID);
@@ -486,8 +599,8 @@ add_task(async function test_user_change_with_disabling() {
 
   is(
     (await Services.search.getDefault()).name,
-    ALTERNATE2_ENGINE_NAME,
-    `Default engine is ${ALTERNATE2_ENGINE_NAME}`
+    ALTERNATE2_ENGINE.name,
+    `Default engine is ${ALTERNATE2_ENGINE.name}`
   );
   await ext1.unload();
   await restoreDefaultEngine();
@@ -499,14 +612,14 @@ add_task(async function test_user_change_with_disabling() {
 add_task(async function test_two_addons_with_first_disabled_before_second() {
   let ext1 = ExtensionTestUtils.loadExtension({
     manifest: {
-      applications: {
+      browser_specific_settings: {
         gecko: {
           id: EXTENSION1_ID,
         },
       },
       chrome_settings_overrides: {
         search_provider: {
-          name: ALTERNATE_ENGINE_NAME,
+          name: ALTERNATE_ENGINE.name,
           search_url: "https://example.com/?q={searchTerms}",
           is_default: true,
         },
@@ -517,14 +630,14 @@ add_task(async function test_two_addons_with_first_disabled_before_second() {
 
   let ext2 = ExtensionTestUtils.loadExtension({
     manifest: {
-      applications: {
+      browser_specific_settings: {
         gecko: {
           id: EXTENSION2_ID,
         },
       },
       chrome_settings_overrides: {
         search_provider: {
-          name: ALTERNATE2_ENGINE_NAME,
+          name: ALTERNATE2_ENGINE.name,
           search_url: "https://example.com/?q={searchTerms}",
           is_default: true,
         },
@@ -538,8 +651,8 @@ add_task(async function test_two_addons_with_first_disabled_before_second() {
 
   is(
     (await Services.search.getDefault()).name,
-    ALTERNATE_ENGINE_NAME,
-    `Default engine is ${ALTERNATE_ENGINE_NAME}`
+    ALTERNATE_ENGINE.name,
+    `Default engine is ${ALTERNATE_ENGINE.name}`
   );
 
   let disabledPromise = awaitEvent("shutdown", EXTENSION1_ID);
@@ -549,8 +662,8 @@ add_task(async function test_two_addons_with_first_disabled_before_second() {
 
   is(
     (await Services.search.getDefault()).name,
-    DEFAULT_ENGINE_NAME,
-    `Default engine is ${DEFAULT_ENGINE_NAME}`
+    DEFAULT_ENGINE.name,
+    `Default engine is ${DEFAULT_ENGINE.name}`
   );
 
   await ext2.startup();
@@ -558,8 +671,8 @@ add_task(async function test_two_addons_with_first_disabled_before_second() {
 
   is(
     (await Services.search.getDefault()).name,
-    ALTERNATE2_ENGINE_NAME,
-    `Default engine is ${ALTERNATE2_ENGINE_NAME}`
+    ALTERNATE2_ENGINE.name,
+    `Default engine is ${ALTERNATE2_ENGINE.name}`
   );
 
   let enabledPromise = awaitEvent("ready", EXTENSION1_ID);
@@ -568,16 +681,16 @@ add_task(async function test_two_addons_with_first_disabled_before_second() {
 
   is(
     (await Services.search.getDefault()).name,
-    ALTERNATE2_ENGINE_NAME,
-    `Default engine is ${ALTERNATE2_ENGINE_NAME}`
+    ALTERNATE2_ENGINE.name,
+    `Default engine is ${ALTERNATE2_ENGINE.name}`
   );
   await ext2.unload();
   await ext1.unload();
 
   is(
     (await Services.search.getDefault()).name,
-    DEFAULT_ENGINE_NAME,
-    `Default engine is ${DEFAULT_ENGINE_NAME}`
+    DEFAULT_ENGINE.name,
+    `Default engine is ${DEFAULT_ENGINE.name}`
   );
 });
 
@@ -587,14 +700,14 @@ add_task(async function test_two_addons_with_first_disabled_before_second() {
 add_task(async function test_two_addons_with_first_disabled() {
   let ext1 = ExtensionTestUtils.loadExtension({
     manifest: {
-      applications: {
+      browser_specific_settings: {
         gecko: {
           id: EXTENSION1_ID,
         },
       },
       chrome_settings_overrides: {
         search_provider: {
-          name: ALTERNATE_ENGINE_NAME,
+          name: ALTERNATE_ENGINE.name,
           search_url: "https://example.com/?q={searchTerms}",
           is_default: true,
         },
@@ -605,14 +718,14 @@ add_task(async function test_two_addons_with_first_disabled() {
 
   let ext2 = ExtensionTestUtils.loadExtension({
     manifest: {
-      applications: {
+      browser_specific_settings: {
         gecko: {
           id: EXTENSION2_ID,
         },
       },
       chrome_settings_overrides: {
         search_provider: {
-          name: ALTERNATE2_ENGINE_NAME,
+          name: ALTERNATE2_ENGINE.name,
           search_url: "https://example.com/?q={searchTerms}",
           is_default: true,
         },
@@ -626,8 +739,8 @@ add_task(async function test_two_addons_with_first_disabled() {
 
   is(
     (await Services.search.getDefault()).name,
-    ALTERNATE_ENGINE_NAME,
-    `Default engine is ${ALTERNATE_ENGINE_NAME}`
+    ALTERNATE_ENGINE.name,
+    `Default engine is ${ALTERNATE_ENGINE.name}`
   );
 
   await ext2.startup();
@@ -635,8 +748,8 @@ add_task(async function test_two_addons_with_first_disabled() {
 
   is(
     (await Services.search.getDefault()).name,
-    ALTERNATE2_ENGINE_NAME,
-    `Default engine is ${ALTERNATE2_ENGINE_NAME}`
+    ALTERNATE2_ENGINE.name,
+    `Default engine is ${ALTERNATE2_ENGINE.name}`
   );
 
   let disabledPromise = awaitEvent("shutdown", EXTENSION1_ID);
@@ -646,8 +759,8 @@ add_task(async function test_two_addons_with_first_disabled() {
 
   is(
     (await Services.search.getDefault()).name,
-    ALTERNATE2_ENGINE_NAME,
-    `Default engine is ${ALTERNATE2_ENGINE_NAME}`
+    ALTERNATE2_ENGINE.name,
+    `Default engine is ${ALTERNATE2_ENGINE.name}`
   );
 
   let enabledPromise = awaitEvent("ready", EXTENSION1_ID);
@@ -656,16 +769,16 @@ add_task(async function test_two_addons_with_first_disabled() {
 
   is(
     (await Services.search.getDefault()).name,
-    ALTERNATE2_ENGINE_NAME,
-    `Default engine is ${ALTERNATE2_ENGINE_NAME}`
+    ALTERNATE2_ENGINE.name,
+    `Default engine is ${ALTERNATE2_ENGINE.name}`
   );
   await ext2.unload();
   await ext1.unload();
 
   is(
     (await Services.search.getDefault()).name,
-    DEFAULT_ENGINE_NAME,
-    `Default engine is ${DEFAULT_ENGINE_NAME}`
+    DEFAULT_ENGINE.name,
+    `Default engine is ${DEFAULT_ENGINE.name}`
   );
 });
 
@@ -675,14 +788,14 @@ add_task(async function test_two_addons_with_first_disabled() {
 add_task(async function test_two_addons_with_second_disabled() {
   let ext1 = ExtensionTestUtils.loadExtension({
     manifest: {
-      applications: {
+      browser_specific_settings: {
         gecko: {
           id: EXTENSION1_ID,
         },
       },
       chrome_settings_overrides: {
         search_provider: {
-          name: ALTERNATE_ENGINE_NAME,
+          name: ALTERNATE_ENGINE.name,
           search_url: "https://example.com/?q={searchTerms}",
           is_default: true,
         },
@@ -693,14 +806,14 @@ add_task(async function test_two_addons_with_second_disabled() {
 
   let ext2 = ExtensionTestUtils.loadExtension({
     manifest: {
-      applications: {
+      browser_specific_settings: {
         gecko: {
           id: EXTENSION2_ID,
         },
       },
       chrome_settings_overrides: {
         search_provider: {
-          name: ALTERNATE2_ENGINE_NAME,
+          name: ALTERNATE2_ENGINE.name,
           search_url: "https://example.com/?q={searchTerms}",
           is_default: true,
         },
@@ -714,8 +827,8 @@ add_task(async function test_two_addons_with_second_disabled() {
 
   is(
     (await Services.search.getDefault()).name,
-    ALTERNATE_ENGINE_NAME,
-    `Default engine is ${ALTERNATE_ENGINE_NAME}`
+    ALTERNATE_ENGINE.name,
+    `Default engine is ${ALTERNATE_ENGINE.name}`
   );
 
   await ext2.startup();
@@ -723,8 +836,8 @@ add_task(async function test_two_addons_with_second_disabled() {
 
   is(
     (await Services.search.getDefault()).name,
-    ALTERNATE2_ENGINE_NAME,
-    `Default engine is ${ALTERNATE2_ENGINE_NAME}`
+    ALTERNATE2_ENGINE.name,
+    `Default engine is ${ALTERNATE2_ENGINE.name}`
   );
 
   let disabledPromise = awaitEvent("shutdown", EXTENSION2_ID);
@@ -734,8 +847,8 @@ add_task(async function test_two_addons_with_second_disabled() {
 
   is(
     (await Services.search.getDefault()).name,
-    ALTERNATE_ENGINE_NAME,
-    `Default engine is ${ALTERNATE_ENGINE_NAME}`
+    ALTERNATE_ENGINE.name,
+    `Default engine is ${ALTERNATE_ENGINE.name}`
   );
 
   let defaultPromise = SearchTestUtils.promiseSearchNotification(
@@ -748,21 +861,21 @@ add_task(async function test_two_addons_with_second_disabled() {
 
   is(
     (await Services.search.getDefault()).name,
-    ALTERNATE2_ENGINE_NAME,
-    `Default engine is ${ALTERNATE2_ENGINE_NAME}`
+    ALTERNATE2_ENGINE.name,
+    `Default engine is ${ALTERNATE2_ENGINE.name}`
   );
   await ext2.unload();
 
   is(
     (await Services.search.getDefault()).name,
-    ALTERNATE_ENGINE_NAME,
-    `Default engine is ${ALTERNATE_ENGINE_NAME}`
+    ALTERNATE_ENGINE.name,
+    `Default engine is ${ALTERNATE_ENGINE.name}`
   );
   await ext1.unload();
 
   is(
     (await Services.search.getDefault()).name,
-    DEFAULT_ENGINE_NAME,
-    `Default engine is ${DEFAULT_ENGINE_NAME}`
+    DEFAULT_ENGINE.name,
+    `Default engine is ${DEFAULT_ENGINE.name}`
   );
 });
