@@ -6,27 +6,39 @@
 
 #![deny(missing_docs)]
 
-use super::*;
+use super::generated::{
+    shorthands, AllShorthand, ComputedValues, LogicalGroupSet, LonghandIdSet,
+    NonCustomPropertyIdSet, PropertyDeclaration, PropertyDeclarationId, PropertyId, ShorthandId,
+    SourcePropertyDeclaration, SourcePropertyDeclarationDrain, SubpropertiesVec,
+};
+use crate::applicable_declarations::CascadePriority;
 use crate::context::QuirksMode;
-use crate::custom_properties::CustomPropertiesBuilder;
+use crate::custom_properties::{self, CustomPropertiesBuilder};
 use crate::error_reporting::{ContextualParseError, ParseErrorReporter};
+use crate::media_queries::Device;
 use crate::parser::ParserContext;
 use crate::properties::animated_properties::{AnimationValue, AnimationValueMap};
+use crate::rule_tree::CascadeLevel;
+use crate::selector_map::PrecomputedHashSet;
 use crate::selector_parser::SelectorImpl;
 use crate::shared_lock::Locked;
 use crate::str::{CssString, CssStringWriter};
-use crate::stylesheets::{CssRuleType, Origin, UrlExtraData};
+use crate::stylesheets::{layer_rule::LayerOrder, CssRuleType, Origin, UrlExtraData};
 use crate::values::computed::Context;
-use cssparser::{parse_important, CowRcStr, DeclarationListParser, ParserInput};
-use cssparser::{AtRuleParser, DeclarationParser, Delimiter, ParseErrorKind, Parser};
+use cssparser::{
+    parse_important, AtRuleParser, CowRcStr, DeclarationParser, Delimiter, ParseErrorKind, Parser,
+    ParserInput, QualifiedRuleParser, RuleBodyItemParser, RuleBodyParser,
+};
 use itertools::Itertools;
 use selectors::SelectorList;
+use servo_arc::Arc;
 use smallbitvec::{self, SmallBitVec};
 use smallvec::SmallVec;
 use std::fmt::{self, Write};
 use std::iter::{DoubleEndedIterator, Zip};
 use std::slice::Iter;
 use style_traits::{CssWriter, ParseError, ParsingMode, StyleParseErrorKind, ToCss};
+use thin_vec::ThinVec;
 
 /// A set of property declarations including animations and transitions.
 #[derive(Default)]
@@ -44,7 +56,6 @@ impl AnimationDeclarations {
     }
 }
 
-
 /// An enum describes how a declaration should update
 /// the declaration block.
 #[derive(Clone, Copy, Debug, Eq, PartialEq)]
@@ -81,29 +92,78 @@ pub enum Importance {
     Important,
 }
 
+impl Default for Importance {
+    fn default() -> Self {
+        Self::Normal
+    }
+}
+
 impl Importance {
     /// Return whether this is an important declaration.
     pub fn important(self) -> bool {
         match self {
-            Importance::Normal => false,
-            Importance::Important => true,
+            Self::Normal => false,
+            Self::Important => true,
         }
     }
 }
 
+#[derive(Clone, ToShmem, Default, MallocSizeOf)]
+struct PropertyDeclarationIdSet {
+    longhands: LonghandIdSet,
+    custom: PrecomputedHashSet<custom_properties::Name>,
+}
+
+impl PropertyDeclarationIdSet {
+    fn insert(&mut self, id: PropertyDeclarationId) -> bool {
+        match id {
+            PropertyDeclarationId::Longhand(id) => {
+                if self.longhands.contains(id) {
+                    return false;
+                }
+                self.longhands.insert(id);
+                return true;
+            },
+            PropertyDeclarationId::Custom(name) => self.custom.insert(name.clone()),
+        }
+    }
+
+    fn contains(&self, id: PropertyDeclarationId) -> bool {
+        match id {
+            PropertyDeclarationId::Longhand(id) => self.longhands.contains(id),
+            PropertyDeclarationId::Custom(name) => self.custom.contains(name),
+        }
+    }
+
+    fn remove(&mut self, id: PropertyDeclarationId) {
+        match id {
+            PropertyDeclarationId::Longhand(id) => self.longhands.remove(id),
+            PropertyDeclarationId::Custom(name) => {
+                self.custom.remove(name);
+            },
+        }
+    }
+
+    fn clear(&mut self) {
+        self.longhands.clear();
+        self.custom.clear();
+    }
+}
+
 /// Overridden declarations are skipped.
 #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
-#[derive(Clone, ToShmem)]
+#[derive(Clone, ToShmem, Default)]
 pub struct PropertyDeclarationBlock {
     /// The group of declarations, along with their importance.
     ///
     /// Only deduplicated declarations appear here.
-    declarations: Vec<PropertyDeclaration>,
+    declarations: ThinVec<PropertyDeclaration>,
 
     /// The "important" flag for each declaration in `declarations`.
     declarations_importance: SmallBitVec,
 
-    longhands: LonghandIdSet,
+    /// The set of properties that are present in the block.
+    property_ids: PropertyDeclarationIdSet,
 }
 
 /// Iterator over `(PropertyDeclaration, Importance)` pairs.
@@ -230,26 +290,32 @@ impl PropertyDeclarationBlock {
         self.declarations.len()
     }
 
+    /// Returns whether the block is empty.
+    #[inline]
+    pub fn is_empty(&self) -> bool {
+        self.declarations.is_empty()
+    }
+
     /// Create an empty block
     #[inline]
     pub fn new() -> Self {
         PropertyDeclarationBlock {
-            declarations: Vec::new(),
+            declarations: ThinVec::new(),
             declarations_importance: SmallBitVec::new(),
-            longhands: LonghandIdSet::new(),
+            property_ids: PropertyDeclarationIdSet::default(),
         }
     }
 
     /// Create a block with a single declaration
     pub fn with_one(declaration: PropertyDeclaration, importance: Importance) -> Self {
-        let mut longhands = LonghandIdSet::new();
-        if let PropertyDeclarationId::Longhand(id) = declaration.id() {
-            longhands.insert(id);
-        }
+        let mut property_ids = PropertyDeclarationIdSet::default();
+        property_ids.insert(declaration.id());
+        let mut declarations = ThinVec::with_capacity(1);
+        declarations.push(declaration);
         PropertyDeclarationBlock {
-            declarations: vec![declaration],
+            declarations,
             declarations_importance: SmallBitVec::from_elem(1, importance.important()),
-            longhands,
+            property_ids,
         }
     }
 
@@ -310,23 +376,23 @@ impl PropertyDeclarationBlock {
         !self.declarations_importance.all_true()
     }
 
-    /// Returns whether this block contains a declaration of a given longhand.
+    /// Returns a `LonghandIdSet` representing the properties that are changed in
+    /// this block.
     #[inline]
-    pub fn contains(&self, id: LonghandId) -> bool {
-        self.longhands.contains(id)
+    pub fn longhands(&self) -> &LonghandIdSet {
+        &self.property_ids.longhands
+    }
+
+    /// Returns whether this block contains a declaration of a given property id.
+    #[inline]
+    pub fn contains(&self, id: PropertyDeclarationId) -> bool {
+        self.property_ids.contains(id)
     }
 
     /// Returns whether this block contains any reset longhand.
     #[inline]
     pub fn contains_any_reset(&self) -> bool {
-        self.longhands.contains_any_reset()
-    }
-
-    /// Returns a `LonghandIdSet` representing the properties that are changed in
-    /// this block.
-    #[inline]
-    pub fn longhands(&self) -> &LonghandIdSet {
-        &self.longhands
+        self.property_ids.longhands.contains_any_reset()
     }
 
     /// Get a declaration for a given property.
@@ -338,12 +404,9 @@ impl PropertyDeclarationBlock {
         &self,
         property: PropertyDeclarationId,
     ) -> Option<(&PropertyDeclaration, Importance)> {
-        if let PropertyDeclarationId::Longhand(id) = property {
-            if !self.contains(id) {
-                return None;
-            }
+        if !self.contains(property) {
+            return None;
         }
-
         self.declaration_importance_iter()
             .find(|(declaration, _)| declaration.id() == property)
     }
@@ -442,16 +505,6 @@ impl PropertyDeclarationBlock {
         }
     }
 
-    /// Returns whether the property is definitely new for this declaration
-    /// block. It returns true when the declaration is a non-custom longhand
-    /// and it doesn't exist in the block, and returns false otherwise.
-    #[inline]
-    fn is_definitely_new(&self, decl: &PropertyDeclaration) -> bool {
-        decl.id()
-            .as_longhand()
-            .map_or(false, |id| !self.longhands.contains(id))
-    }
-
     /// Adds or overrides the declaration for a given property in this block.
     ///
     /// See the documentation of `push` to see what impact `source` has when the
@@ -490,10 +543,11 @@ impl PropertyDeclarationBlock {
     ///
     /// This is only used for parsing and internal use.
     pub fn push(&mut self, declaration: PropertyDeclaration, importance: Importance) -> bool {
-        if !self.is_definitely_new(&declaration) {
+        let id = declaration.id();
+        if !self.property_ids.insert(id) {
             let mut index_to_remove = None;
             for (i, slot) in self.declarations.iter_mut().enumerate() {
-                if slot.id() != declaration.id() {
+                if slot.id() != id {
                     continue;
                 }
 
@@ -518,9 +572,6 @@ impl PropertyDeclarationBlock {
             }
         }
 
-        if let PropertyDeclarationId::Longhand(id) = declaration.id() {
-            self.longhands.insert(id);
-        }
         self.declarations.push(declaration);
         self.declarations_importance.push(importance.important());
         true
@@ -543,7 +594,7 @@ impl PropertyDeclarationBlock {
                 .all_shorthand
                 .declarations()
                 .any(|decl| {
-                    self.is_definitely_new(&decl) ||
+                    !self.contains(decl.id()) ||
                         self.declarations
                             .iter()
                             .enumerate()
@@ -564,7 +615,7 @@ impl PropertyDeclarationBlock {
                 .declarations
                 .iter()
                 .map(|declaration| {
-                    if self.is_definitely_new(declaration) {
+                    if !self.contains(declaration.id()) {
                         return DeclarationUpdate::Append;
                     }
                     let longhand_id = declaration.id().as_longhand();
@@ -639,11 +690,10 @@ impl PropertyDeclarationBlock {
         if !matches!(drain.all_shorthand, AllShorthand::NotSet) {
             debug_assert!(updates.updates.is_empty());
             for decl in drain.all_shorthand.declarations() {
-                if self.is_definitely_new(&decl) {
-                    let longhand_id = decl.id().as_longhand().unwrap();
+                let id = decl.id();
+                if self.property_ids.insert(id) {
                     self.declarations.push(decl);
                     self.declarations_importance.push(important);
-                    self.longhands.insert(longhand_id);
                 } else {
                     let (idx, slot) = self
                         .declarations
@@ -710,9 +760,7 @@ impl PropertyDeclarationBlock {
             match *update {
                 DeclarationUpdate::None => {},
                 DeclarationUpdate::Append | DeclarationUpdate::AppendAndRemove { .. } => {
-                    if let Some(id) = decl.id().as_longhand() {
-                        self.longhands.insert(id);
-                    }
+                    self.property_ids.insert(decl.id());
                     self.declarations.push(decl);
                     self.declarations_importance.push(important);
                 },
@@ -729,8 +777,8 @@ impl PropertyDeclarationBlock {
     /// `property`.
     #[inline]
     pub fn first_declaration_to_remove(&self, property: &PropertyId) -> Option<usize> {
-        if let Some(id) = property.longhand_id() {
-            if !self.longhands.contains(id) {
+        if let Err(longhand_or_custom) = property.as_shorthand() {
+            if !self.contains(longhand_or_custom) {
                 return None;
             }
         }
@@ -743,16 +791,19 @@ impl PropertyDeclarationBlock {
     /// Removes a given declaration at a given index.
     #[inline]
     fn remove_declaration_at(&mut self, i: usize) {
-        {
-            let id = self.declarations[i].id();
-            if let PropertyDeclarationId::Longhand(id) = id {
-                self.longhands.remove(id);
-            }
-            self.declarations_importance.remove(i);
-        }
+        self.property_ids.remove(self.declarations[i].id());
+        self.declarations_importance.remove(i);
         self.declarations.remove(i);
     }
 
+    /// Clears all the declarations from this block.
+    #[inline]
+    pub fn clear(&mut self) {
+        self.declarations_importance.clear();
+        self.declarations.clear();
+        self.property_ids.clear();
+    }
+
     /// <https://drafts.csswg.org/cssom/#dom-cssstyledeclaration-removeproperty>
     ///
     /// `first_declaration` needs to be the result of
@@ -838,7 +889,7 @@ impl PropertyDeclarationBlock {
                         custom_properties.as_ref(),
                         QuirksMode::NoQuirks,
                         device,
-                        &mut Default::default()
+                        &mut Default::default(),
                     )
                     .to_css(dest)
             },
@@ -849,17 +900,17 @@ impl PropertyDeclarationBlock {
     /// Convert AnimationValueMap to PropertyDeclarationBlock.
     pub fn from_animation_value_map(animation_value_map: &AnimationValueMap) -> Self {
         let len = animation_value_map.len();
-        let mut declarations = Vec::with_capacity(len);
-        let mut longhands = LonghandIdSet::new();
+        let mut declarations = ThinVec::with_capacity(len);
+        let mut property_ids = PropertyDeclarationIdSet::default();
 
         for (property, animation_value) in animation_value_map.iter() {
-            longhands.insert(*property);
+            property_ids.longhands.insert(*property);
             declarations.push(animation_value.uncompute());
         }
 
         PropertyDeclarationBlock {
             declarations,
-            longhands,
+            property_ids,
             declarations_importance: SmallBitVec::from_elem(len, false),
         }
     }
@@ -867,8 +918,8 @@ impl PropertyDeclarationBlock {
     /// Returns true if the declaration block has a CSSWideKeyword for the given
     /// property.
     pub fn has_css_wide_keyword(&self, property: &PropertyId) -> bool {
-        if let Some(id) = property.longhand_id() {
-            if !self.longhands.contains(id) {
+        if let Err(longhand_or_custom) = property.as_shorthand() {
+            if !self.property_ids.contains(longhand_or_custom) {
                 return false;
             }
         }
@@ -884,10 +935,7 @@ impl PropertyDeclarationBlock {
         &self,
         context: &Context,
     ) -> Option<Arc<crate::custom_properties::CustomPropertiesMap>> {
-        self.cascade_custom_properties(
-            context.style().custom_properties(),
-            context.device(),
-        )
+        self.cascade_custom_properties(context.style().custom_properties(), context.device())
     }
 
     /// Returns a custom properties map which is the result of cascading custom
@@ -902,7 +950,13 @@ impl PropertyDeclarationBlock {
 
         for declaration in self.normal_declaration_iter() {
             if let PropertyDeclaration::Custom(ref declaration) = *declaration {
-                builder.cascade(declaration, Origin::Author);
+                builder.cascade(
+                    declaration,
+                    CascadePriority::new(
+                        CascadeLevel::same_tree_author_normal(),
+                        LayerOrder::root(),
+                    ),
+                );
             }
         }
 
@@ -987,7 +1041,7 @@ impl PropertyDeclarationBlock {
                 //     If all properties that map to shorthand are not present
                 //     in longhands, continue with the steps labeled shorthand
                 //     loop.
-                if !self.longhands.contains_all(&longhands) {
+                if !self.property_ids.longhands.contains_all(&longhands) {
                     continue;
                 }
 
@@ -1066,12 +1120,11 @@ impl PropertyDeclarationBlock {
                 // 3.4.7:
                 //    Let value be the result of invoking serialize a CSS value
                 //    of current longhands.
-                let appendable_value = match shorthand
-                    .get_shorthand_appendable_value(&current_longhands)
-                {
-                    None => continue,
-                    Some(appendable_value) => appendable_value,
-                };
+                let appendable_value =
+                    match shorthand.get_shorthand_appendable_value(&current_longhands) {
+                        None => continue,
+                        Some(appendable_value) => appendable_value,
+                    };
 
                 // We avoid re-serializing if we're already an
                 // AppendableValue::Css.
@@ -1094,7 +1147,9 @@ impl PropertyDeclarationBlock {
                         AppendableValue::Css({
                             // Safety: serialization only generates valid utf-8.
                             #[cfg(feature = "gecko")]
-                            unsafe { v.as_str_unchecked() }
+                            unsafe {
+                                v.as_str_unchecked()
+                            }
                             #[cfg(feature = "servo")]
                             &v
                         })
@@ -1185,7 +1240,7 @@ where
     W: Write,
 {
     if !*is_first_serialization {
-        dest.write_str(" ")
+        dest.write_char(' ')
     } else {
         *is_first_serialization = false;
         Ok(())
@@ -1249,6 +1304,7 @@ pub fn parse_style_attribute(
         Some(rule_type),
         ParsingMode::DEFAULT,
         quirks_mode,
+        /* namespaces = */ Default::default(),
         error_reporter,
         None,
     );
@@ -1279,6 +1335,7 @@ pub fn parse_one_declaration_into(
         Some(rule_type),
         parsing_mode,
         quirks_mode,
+        /* namespaces = */ Default::default(),
         error_reporter,
         None,
     );
@@ -1311,18 +1368,128 @@ pub fn parse_one_declaration_into(
 }
 
 /// A struct to parse property declarations.
-struct PropertyDeclarationParser<'a, 'b: 'a> {
+struct PropertyDeclarationParser<'a, 'b: 'a, 'i> {
     context: &'a ParserContext<'b>,
-    declarations: &'a mut SourcePropertyDeclaration,
-    /// The last parsed property id if any.
+    state: &'a mut DeclarationParserState<'i>,
+}
+
+/// The state needed to parse a declaration block.
+///
+/// It stores declarations in output_block.
+#[derive(Default)]
+pub struct DeclarationParserState<'i> {
+    /// The output block where results are stored.
+    output_block: PropertyDeclarationBlock,
+    /// Declarations from the last declaration parsed. (note that a shorthand might expand to
+    /// multiple declarations).
+    declarations: SourcePropertyDeclaration,
+    /// The importance from the last declaration parsed.
+    importance: Importance,
+    /// A list of errors that have happened so far. Not all of them might be reported.
+    errors: SmallParseErrorVec<'i>,
+    /// The last parsed property id, if any.
     last_parsed_property_id: Option<PropertyId>,
 }
 
+impl<'i> DeclarationParserState<'i> {
+    /// Returns whether any parsed declarations have been parsed so far.
+    pub fn has_parsed_declarations(&self) -> bool {
+        !self.output_block.is_empty()
+    }
+
+    /// Takes the parsed declarations.
+    pub fn take_declarations(&mut self) -> PropertyDeclarationBlock {
+        std::mem::take(&mut self.output_block)
+    }
+
+    /// Parse a single declaration value.
+    pub fn parse_value<'t>(
+        &mut self,
+        context: &ParserContext,
+        name: CowRcStr<'i>,
+        input: &mut Parser<'i, 't>,
+    ) -> Result<(), ParseError<'i>> {
+        let id = match PropertyId::parse(&name, context) {
+            Ok(id) => id,
+            Err(..) => {
+                return Err(input.new_custom_error(StyleParseErrorKind::UnknownProperty(name)));
+            },
+        };
+        if context.error_reporting_enabled() {
+            self.last_parsed_property_id = Some(id.clone());
+        }
+        input.parse_until_before(Delimiter::Bang, |input| {
+            PropertyDeclaration::parse_into(&mut self.declarations, id, context, input)
+        })?;
+        self.importance = match input.try_parse(parse_important) {
+            Ok(()) => Importance::Important,
+            Err(_) => Importance::Normal,
+        };
+        // In case there is still unparsed text in the declaration, we should roll back.
+        input.expect_exhausted()?;
+        self.output_block
+            .extend(self.declarations.drain(), self.importance);
+        // We've successfully parsed a declaration, so forget about
+        // `last_parsed_property_id`. It'd be wrong to associate any
+        // following error with this property.
+        self.last_parsed_property_id = None;
+        Ok(())
+    }
+
+    /// Reports any CSS errors that have ocurred if needed.
+    #[inline]
+    pub fn report_errors_if_needed(
+        &mut self,
+        context: &ParserContext,
+        selectors: Option<&SelectorList<SelectorImpl>>,
+    ) {
+        if self.errors.is_empty() {
+            return;
+        }
+        self.do_report_css_errors(context, selectors);
+    }
+
+    #[cold]
+    fn do_report_css_errors(
+        &mut self,
+        context: &ParserContext,
+        selectors: Option<&SelectorList<SelectorImpl>>,
+    ) {
+        for (error, slice, property) in self.errors.drain(..) {
+            report_one_css_error(
+                context,
+                Some(&self.output_block),
+                selectors,
+                error,
+                slice,
+                property,
+            )
+        }
+    }
+
+    /// Resets the declaration parser state, and reports the error if needed.
+    #[inline]
+    pub fn did_error(&mut self, context: &ParserContext, error: ParseError<'i>, slice: &'i str) {
+        self.declarations.clear();
+        if !context.error_reporting_enabled() {
+            return;
+        }
+        let property = self.last_parsed_property_id.take();
+        self.errors.push((error, slice, property));
+    }
+}
+
 /// Default methods reject all at rules.
-impl<'a, 'b, 'i> AtRuleParser<'i> for PropertyDeclarationParser<'a, 'b> {
-    type PreludeNoBlock = ();
-    type PreludeBlock = ();
-    type AtRule = Importance;
+impl<'a, 'b, 'i> AtRuleParser<'i> for PropertyDeclarationParser<'a, 'b, 'i> {
+    type Prelude = ();
+    type AtRule = ();
+    type Error = StyleParseErrorKind<'i>;
+}
+
+/// Default methods reject all rules.
+impl<'a, 'b, 'i> QualifiedRuleParser<'i> for PropertyDeclarationParser<'a, 'b, 'i> {
+    type Prelude = ();
+    type QualifiedRule = ();
     type Error = StyleParseErrorKind<'i>;
 }
 
@@ -1331,34 +1498,28 @@ fn is_non_mozilla_vendor_identifier(name: &str) -> bool {
     (name.starts_with("-") && !name.starts_with("-moz-")) || name.starts_with("_")
 }
 
-impl<'a, 'b, 'i> DeclarationParser<'i> for PropertyDeclarationParser<'a, 'b> {
-    type Declaration = Importance;
+impl<'a, 'b, 'i> DeclarationParser<'i> for PropertyDeclarationParser<'a, 'b, 'i> {
+    type Declaration = ();
     type Error = StyleParseErrorKind<'i>;
 
     fn parse_value<'t>(
         &mut self,
         name: CowRcStr<'i>,
         input: &mut Parser<'i, 't>,
-    ) -> Result<Importance, ParseError<'i>> {
-        let id = match PropertyId::parse(&name, self.context) {
-            Ok(id) => id,
-            Err(..) => {
-                return Err(input.new_custom_error(StyleParseErrorKind::UnknownProperty(name)));
-            },
-        };
-        if self.context.error_reporting_enabled() {
-            self.last_parsed_property_id = Some(id.clone());
-        }
-        input.parse_until_before(Delimiter::Bang, |input| {
-            PropertyDeclaration::parse_into(self.declarations, id, self.context, input)
-        })?;
-        let importance = match input.try_parse(parse_important) {
-            Ok(()) => Importance::Important,
-            Err(_) => Importance::Normal,
-        };
-        // In case there is still unparsed text in the declaration, we should roll back.
-        input.expect_exhausted()?;
-        Ok(importance)
+    ) -> Result<(), ParseError<'i>> {
+        self.state.parse_value(self.context, name, input)
+    }
+}
+
+impl<'a, 'b, 'i> RuleBodyItemParser<'i, (), StyleParseErrorKind<'i>>
+    for PropertyDeclarationParser<'a, 'b, 'i>
+{
+    fn parse_declarations(&self) -> bool {
+        true
+    }
+    // TODO(emilio): Nesting.
+    fn parse_qualified(&self) -> bool {
+        false
     }
 }
 
@@ -1383,15 +1544,11 @@ fn report_one_css_error<'i>(
     debug_assert!(context.error_reporting_enabled());
 
     fn all_properties_in_block(block: &PropertyDeclarationBlock, property: &PropertyId) -> bool {
-        match *property {
-            PropertyId::LonghandAlias(id, _) | PropertyId::Longhand(id) => block.contains(id),
-            PropertyId::ShorthandAlias(id, _) | PropertyId::Shorthand(id) => {
-                id.longhands().all(|longhand| block.contains(longhand))
-            },
-            // NOTE(emilio): We could do this, but it seems of limited utility,
-            // and it's linear on the size of the declaration block, so let's
-            // not.
-            PropertyId::Custom(..) => false,
+        match property.as_shorthand() {
+            Ok(id) => id
+                .longhands()
+                .all(|longhand| block.contains(PropertyDeclarationId::Longhand(longhand))),
+            Err(longhand_or_custom) => block.contains(longhand_or_custom),
         }
     }
 
@@ -1432,18 +1589,6 @@ fn report_one_css_error<'i>(
     context.log_css_error(location, error);
 }
 
-#[cold]
-fn report_css_errors(
-    context: &ParserContext,
-    block: &PropertyDeclarationBlock,
-    selectors: Option<&SelectorList<SelectorImpl>>,
-    errors: &mut SmallParseErrorVec,
-) {
-    for (error, slice, property) in errors.drain(..) {
-        report_one_css_error(context, Some(block), selectors, error, slice, property)
-    }
-}
-
 /// Parse a list of property declarations and return a property declaration
 /// block.
 pub fn parse_property_declaration_list(
@@ -1451,38 +1596,18 @@ pub fn parse_property_declaration_list(
     input: &mut Parser,
     selectors: Option<&SelectorList<SelectorImpl>>,
 ) -> PropertyDeclarationBlock {
-    let mut declarations = SourcePropertyDeclaration::new();
-    let mut block = PropertyDeclarationBlock::new();
-    let parser = PropertyDeclarationParser {
+    let mut state = DeclarationParserState::default();
+    let mut parser = PropertyDeclarationParser {
         context,
-        last_parsed_property_id: None,
-        declarations: &mut declarations,
+        state: &mut state,
     };
-    let mut iter = DeclarationListParser::new(input, parser);
-    let mut errors = SmallParseErrorVec::new();
+    let mut iter = RuleBodyParser::new(input, &mut parser);
     while let Some(declaration) = iter.next() {
         match declaration {
-            Ok(importance) => {
-                block.extend(iter.parser.declarations.drain(), importance);
-                // We've successfully parsed a declaration, so forget about
-                // `last_parsed_property_id`. It'd be wrong to associate any
-                // following error with this property.
-                iter.parser.last_parsed_property_id = None;
-            },
-            Err((error, slice)) => {
-                iter.parser.declarations.clear();
-
-                if context.error_reporting_enabled() {
-                    let property = iter.parser.last_parsed_property_id.take();
-                    errors.push((error, slice, property));
-                }
-            },
+            Ok(()) => {},
+            Err((error, slice)) => iter.parser.state.did_error(context, error, slice),
         }
     }
-
-    if !errors.is_empty() {
-        report_css_errors(context, &block, selectors, &mut errors)
-    }
-
-    block
+    parser.state.report_errors_if_needed(context, selectors);
+    state.output_block
 }
