@@ -138,7 +138,7 @@
             self.row_gap.to_css(dest)
           } else {
             self.row_gap.to_css(dest)?;
-            dest.write_str(" ")?;
+            dest.write_char(' ')?;
             self.column_gap.to_css(dest)
           }
       }
@@ -497,7 +497,7 @@
                                                                   .zip(&mut names_iter)
                                                                   .zip(track_list.values.iter()) {
                     if i > 0 {
-                        dest.write_str(" ")?;
+                        dest.write_char(' ')?;
                     }
 
                     if !names.is_empty() {
@@ -508,7 +508,7 @@
 
                     // If the track size is the initial value then it's redundant here.
                     if !value.is_initial() {
-                        dest.write_str(" ")?;
+                        dest.write_char(' ')?;
                         value.to_css(dest)?;
                     }
                 }
@@ -600,7 +600,7 @@
             temp_rows = rows;
             input.expect_delim('/')?;
             flow = parse_auto_flow(input, false)?;
-            auto_cols = grid_auto_columns::parse(context, input).unwrap_or_default();
+            auto_cols = input.try_parse(|i| grid_auto_columns::parse(context, i)).unwrap_or_default();
         } else {
             flow = parse_auto_flow(input, true)?;
             auto_rows = input.try_parse(|i| grid_auto_rows::parse(context, i)).unwrap_or_default();
@@ -621,7 +621,6 @@
     impl<'a> LonghandsToSerialize<'a> {
         /// Returns true if other sub properties except template-{rows,columns} are initial.
         fn is_grid_template(&self) -> bool {
-            *self.grid_template_areas == GridTemplateAreas::None &&
             self.grid_auto_rows.is_initial() &&
             self.grid_auto_columns.is_initial() &&
             *self.grid_auto_flow == grid_auto_flow::get_initial_value()
@@ -630,13 +629,19 @@
 
     impl<'a> ToCss for LonghandsToSerialize<'a> {
         fn to_css<W>(&self, dest: &mut CssWriter<W>) -> fmt::Result where W: fmt::Write {
-            if *self.grid_template_areas != GridTemplateAreas::None ||
-               (!self.grid_template_rows.is_initial() &&
-                !self.grid_template_columns.is_initial()) ||
-               self.is_grid_template() {
-                return super::grid_template::serialize_grid_template(self.grid_template_rows,
-                                                                     self.grid_template_columns,
-                                                                     self.grid_template_areas, dest);
+            if self.is_grid_template() {
+                return super::grid_template::serialize_grid_template(
+                    self.grid_template_rows,
+                    self.grid_template_columns,
+                    self.grid_template_areas,
+                    dest
+                );
+            }
+
+            if *self.grid_template_areas != GridTemplateAreas::None {
+                // No other syntax can set the template areas, so fail to
+                // serialize.
+                return Ok(());
             }
 
             if self.grid_auto_flow.contains(GridAutoFlow::COLUMN) {
@@ -660,36 +665,38 @@
                 }
 
                 if !self.grid_auto_columns.is_initial() {
-                    dest.write_str(" ")?;
+                    dest.write_char(' ')?;
                     self.grid_auto_columns.to_css(dest)?;
                 }
-            } else {
-                // It should fail to serialize if other branch of the if condition's values are set.
-                if !self.grid_auto_columns.is_initial() ||
-                    !self.grid_template_rows.is_initial() {
+
+                return Ok(());
+            }
+
+            // It should fail to serialize if other branch of the if condition's values are set.
+            if !self.grid_auto_columns.is_initial() ||
+                !self.grid_template_rows.is_initial() {
+                return Ok(());
+            }
+
+            // It should fail to serialize if template-column value is not Explicit.
+            if let GenericGridTemplateComponent::TrackList(ref list) = *self.grid_template_columns {
+                if !list.is_explicit() {
                     return Ok(());
                 }
-
-                // It should fail to serialize if template-column value is not Explicit.
-                if let GenericGridTemplateComponent::TrackList(ref list) = *self.grid_template_columns {
-                    if !list.is_explicit() {
-                        return Ok(());
-                    }
-                }
-
-                dest.write_str("auto-flow")?;
-                if self.grid_auto_flow.contains(GridAutoFlow::DENSE) {
-                    dest.write_str(" dense")?;
-                }
-
-                if !self.grid_auto_rows.is_initial() {
-                    dest.write_str(" ")?;
-                    self.grid_auto_rows.to_css(dest)?;
-                }
-
-                dest.write_str(" / ")?;
-                self.grid_template_columns.to_css(dest)?;
             }
+
+            dest.write_str("auto-flow")?;
+            if self.grid_auto_flow.contains(GridAutoFlow::DENSE) {
+                dest.write_str(" dense")?;
+            }
+
+            if !self.grid_auto_rows.is_initial() {
+                dest.write_char(' ')?;
+                self.grid_auto_rows.to_css(dest)?;
+            }
+
+            dest.write_str(" / ")?;
+            self.grid_template_columns.to_css(dest)?;
             Ok(())
         }
     }
@@ -742,7 +749,7 @@
         fn to_css<W>(&self, dest: &mut CssWriter<W>) -> fmt::Result where W: fmt::Write {
             self.align_content.to_css(dest)?;
             if self.align_content.0 != self.justify_content.0 {
-                dest.write_str(" ")?;
+                dest.write_char(' ')?;
                 self.justify_content.to_css(dest)?;
             }
             Ok(())
@@ -783,7 +790,7 @@
         fn to_css<W>(&self, dest: &mut CssWriter<W>) -> fmt::Result where W: fmt::Write {
             self.align_self.to_css(dest)?;
             if self.align_self.0 != self.justify_self.0 {
-                dest.write_str(" ")?;
+                dest.write_char(' ')?;
                 self.justify_self.to_css(dest)?;
             }
             Ok(())
@@ -825,7 +832,7 @@
         fn to_css<W>(&self, dest: &mut CssWriter<W>) -> fmt::Result where W: fmt::Write {
             self.align_items.to_css(dest)?;
             if self.align_items.0 != self.justify_items.0 {
-                dest.write_str(" ")?;
+                dest.write_char(' ')?;
                 self.justify_items.to_css(dest)?;
             }
 
@@ -861,3 +868,12 @@ ${helpers.two_properties_shorthand(
     engines="gecko servo-2013",
     spec="https://drafts.csswg.org/css-logical/#propdef-inset-inline"
 )}
+
+${helpers.two_properties_shorthand(
+    "contain-intrinsic-size",
+    "contain-intrinsic-width",
+    "contain-intrinsic-height",
+    engines="gecko",
+    gecko_pref="layout.css.contain-intrinsic-size.enabled",
+    spec="https://drafts.csswg.org/css-sizing-4/#intrinsic-size-override",
+)}
